# -*- coding: utf-8 -*-
# Copyright 2019 Bruno Toshio Sugano <brunotoshio@gmail.com>

import logging

from .jconverter import alpha_to_full
from .jconverter import digits_to_half
from .jconverter import kana_to_full
from .replacer import lower
from .replacer import replace_emails
from .replacer import replace_hashtags
from .replacer import replace_mentions
from .replacer import replace_numbers
from .replacer import replace_prices
from .replacer import replace_urls


class Kotonoha(object):

    def __init__(self):
        self._pipeline = []
        self._operators = {
            'remove_url': replace_urls,
            'replace_url': replace_urls,
            'remove_prices': replace_prices,
            'replace_prices': replace_prices,
            'remove_numbers': replace_numbers,
            'replace_numbers': replace_numbers,
            'to_full_width': kana_to_full,
            'digits': digits_to_half,
            'alpha_to_full': alpha_to_full,
            'lower': lower,
            'replace_hashtags': replace_hashtags,
            'replace_emails': replace_emails,
            'replace_mentions': replace_mentions
        }

    def prepare(self, pipeline):
        self._pipeline = []
        for step in pipeline:
            task = {}
            operation = next(iter(step))
            if operation in self._operators:
                task['handler'] = self._operators[operation]
                if isinstance(step, dict):
                    task['args'] = step[operation]
                self._pipeline.append(task)
            else:
                logging.error(f'Invalid operation: {operation}')
                return

    def run(self, text):
        next_input = text
        for task in self._pipeline:
            if 'args' in task:
                args = task['args']
                next_input = task['handler'](next_input, **args)
            else:
                next_input = task['handler'](next_input)
        return next_input



# -*- coding: utf-8 -*-
# Copyright 2019 Bruno Toshio Sugano <brunotoshio@gmail.com>


class MeCabHandler:

    def __init__(self, tagger):
        self._handler = tagger

    def basic(self, text):
        self._handler.parse("")
        result = self._handler.parseToNode(text)
        filtered_words = []
        while result:
            features = result.feature.split(',')
            if features[0] == u"名詞":
                filtered_words.append(result.surface)
            else:
                filtered_words.append(features[6])
            result = result.next

        return " ".join(filtered_words)

    def meaningful(self, text):
        self._handler.parse("")
        result = self._handler.parseToNode(text)
        filtered_words = []
        while result:
            features = result.feature.split(',')
            if features[0] in {"形容詞"}:
                filtered_words.append(features[6])
            if features[0] in {"動詞"} and features[1] == "自立":
                filtered_words.append(features[6])
            if features[0] in {"名詞"} and features[1] not in {"数", "接尾"}:
                filtered_words.append(result.surface)
            result = result.next

        return " ".join(filtered_words)

    def nouns(self, text):
        self._handler.parse("")
        result = self._handler.parseToNode(text)
        filtered_words = []
        while result:
            features = result.feature.split(',')
            if features[0] in {"名詞"}:
                filtered_words.append(result.surface)
            result = result.next

        return " ".join(filtered_words)

    def verbs(self, text):
        self._handler.parse("")
        result = self._handler.parseToNode(text)
        filtered_words = []
        while result:
            features = result.feature.split(',')
            if features[0] in {"動詞"} and features[1] == "自立":
                filtered_words.append(features[6])
            if features[0] in {"名詞"} and features[1] in {"サ変接続"}:
                filtered_words.append(result.surface)
            result = result.next

        return " ".join(filtered_words)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
from kotonoha import jconverter


def test_kana_to_full():
    assert jconverter.kana_to_full('これはｲｲABC1234') == 'これはイイABC1234'


def test_digits_to_half():
    assert jconverter.digits_to_half('今日は３９度みたいです1234ABC') == '今日は39度みたいです1234ABC'


def test_alpha_to_full():
    assert jconverter.alpha_to_full('takaidesu') == 'たかいです'



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
import logging
from unittest import mock

from kotonoha import kotonoha


@mock.patch('kotonoha.kotonoha.alpha_to_full')
def test_class_jtext_alpha_to_full(mocked):
    jtext = kotonoha.Kotonoha()
    jtext.prepare([
        {'alpha_to_full'}
    ])
    jtext.run('kore')
    assert mocked.called


@mock.patch('kotonoha.kotonoha.digits_to_half')
def test_class_jtext_digits(mocked):
    jtext = kotonoha.Kotonoha()
    jtext.prepare([
        {'digits'}
    ])
    jtext.run('kore')
    assert mocked.called


@mock.patch('kotonoha.kotonoha.kana_to_full')
def test_class_jtext_to_full_width(mocked):
    jtext = kotonoha.Kotonoha()
    jtext.prepare([
        {'to_full_width'}
    ])
    jtext.run('kore')
    assert mocked.called


@mock.patch('kotonoha.kotonoha.lower')
def test_class_jtext_lower(mocked):
    jtext = kotonoha.Kotonoha()
    jtext.prepare([
        {'lower'}
    ])
    jtext.run('kore')
    assert mocked.called


@mock.patch('kotonoha.kotonoha.replace_numbers')
def test_class_jtext_replace_numbers(mocked):
    jtext = kotonoha.Kotonoha()
    jtext.prepare([
        {'replace_numbers': {'replace_text': 'n'}}
    ])
    jtext.run('kore')
    assert mocked.called


@mock.patch('kotonoha.kotonoha.replace_numbers')
def test_class_jtext_remove_numbers(mocked):
    jtext = kotonoha.Kotonoha()
    jtext.prepare([
        {'remove_numbers'}
    ])
    jtext.run('kore')
    assert mocked.called


@mock.patch('kotonoha.kotonoha.replace_prices')
def test_class_jtext_replace_prices(mocked):
    jtext = kotonoha.Kotonoha()
    jtext.prepare([
        {'replace_prices': {'replace_text': '$'}}
    ])
    jtext.run('kore')
    assert mocked.called


@mock.patch('kotonoha.kotonoha.replace_prices')
def test_class_jtext_remove_prices(mocked):
    jtext = kotonoha.Kotonoha()
    jtext.prepare([
        {'remove_prices'}
    ])
    jtext.run('kore')
    assert mocked.called


@mock.patch('kotonoha.kotonoha.replace_urls')
def test_class_jtext_replace_url(mocked):
    jtext = kotonoha.Kotonoha()
    jtext.prepare([
        {'replace_url': {'replace_text': 'url'}}
    ])
    jtext.run('kore')
    assert mocked.called


@mock.patch('kotonoha.kotonoha.replace_urls')
def test_class_jtext_remove_url(mocked):
    jtext = kotonoha.Kotonoha()
    jtext.prepare([
        {'remove_url'}
    ])
    jtext.run('kore')
    assert mocked.called


@mock.patch('kotonoha.kotonoha.replace_hashtags')
def test_class_jtext_replace_hashtag(mocked):
    jtext = kotonoha.Kotonoha()
    jtext.prepare([
        {'replace_hashtags'}
    ])
    jtext.run('kore')
    assert mocked.called


@mock.patch('kotonoha.kotonoha.replace_emails')
def test_class_jtext_replace_email(mocked):
    jtext = kotonoha.Kotonoha()
    jtext.prepare([
        {'replace_emails'}
    ])
    jtext.run('kore')
    assert mocked.called


@mock.patch('kotonoha.kotonoha.replace_mentions')
def test_class_jtext_replace_mention(mocked):
    jtext = kotonoha.Kotonoha()
    jtext.prepare([
        {'replace_mentions'}
    ])
    jtext.run('kore')
    assert mocked.called


def test_class_jtext_prepare():
    jtext = kotonoha.Kotonoha()
    list_of_tasks = [
        {'alpha_to_full'},
        {'digits'},
        {'to_full_width'},
        {'lower'},
        {'replace_numbers': {'replace_text': 'n'}},
        {'remove_numbers'},
        {'replace_prices': {'replace_text': '$'}},
        {'remove_prices'},
        {'replace_url': {'replace_text': 'url'}},
        {'remove_url'},
        {'replace_hashtags': {'replace_text': 'hash'}},
        {'replace_emails': {'replace_text': 'email'}},
        {'replace_mentions': {'replace_text': 'mention'}}
    ]
    assert len(list_of_tasks) == len(jtext._operators.keys())
    jtext.prepare(list_of_tasks)
    assert len(jtext._pipeline) == len(list_of_tasks)


def test_class_jtext_prepare_error(caplog):
    with caplog.at_level(logging.ERROR):
        jtext = kotonoha.Kotonoha()
        jtext.prepare([{'non_existing_task'}])
        assert ['Invalid operation: non_existing_task'] == [rec.message for rec in caplog.records]



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
import pytest

from kotonoha.mecab_handler import MeCabHandler


class Struct:
    def __init__(self, **entries):
        self.__dict__.update(entries)


class MecabNode:
    def __init__(self, feature):
        self.feature = feature
        self.surface = "aaa"
        self.next = None


@pytest.fixture
def mecab_mocker():
    def mock_parse(text):
        return text

    def mock_parse_node(text):
        nodes = [
            '形容詞,,,,,,bla',
            '動詞,自立,,,,,bla',
            '動詞,,,,,,bla',
            '名詞,サ変接続,,,,,bla',
            '名詞,数,,,,,bla',
            '名詞,接尾,,,,,bla',
        ]
        initial = None
        for node in nodes:
            temp = MecabNode(node)
            temp.next = initial
            initial = temp
        return initial

    mocker = {}
    mocker['parse'] = mock_parse
    mocker['parseToNode'] = mock_parse_node
    return Struct(**mocker)


def test_basic(mecab_mocker, mocker):
    handler = MeCabHandler(mecab_mocker)
    print(mecab_mocker.parseToNode('111'))
    mocker.spy(mecab_mocker, 'parse')
    mocker.spy(mecab_mocker, 'parseToNode')
    handler.basic('aaa')
    assert mecab_mocker.parse.call_count == 1
    assert mecab_mocker.parseToNode.call_count == 1


def test_meaningful(mecab_mocker, mocker):
    handler = MeCabHandler(mecab_mocker)
    print(mecab_mocker.parseToNode('111'))
    mocker.spy(mecab_mocker, 'parse')
    mocker.spy(mecab_mocker, 'parseToNode')
    handler.meaningful('aaa')
    assert mecab_mocker.parse.call_count == 1
    assert mecab_mocker.parseToNode.call_count == 1


def test_nouns(mecab_mocker, mocker):
    handler = MeCabHandler(mecab_mocker)
    print(mecab_mocker.parseToNode('111'))
    mocker.spy(mecab_mocker, 'parse')
    mocker.spy(mecab_mocker, 'parseToNode')
    handler.nouns('aaa')
    assert mecab_mocker.parse.call_count == 1
    assert mecab_mocker.parseToNode.call_count == 1


def test_verbs(mecab_mocker, mocker):
    handler = MeCabHandler(mecab_mocker)
    print(mecab_mocker.parseToNode('111'))
    mocker.spy(mecab_mocker, 'parse')
    mocker.spy(mecab_mocker, 'parseToNode')
    handler.verbs('aaa')
    assert mecab_mocker.parse.call_count == 1
    assert mecab_mocker.parseToNode.call_count == 1



# -*- coding: utf-8 -*-
# Copyright 2019 Bruno Toshio Sugano <brunotoshio@gmail.com>

import jaconv


def kana_to_full(text):
    return jaconv.h2z(text, ascii=False, digit=False)


def digits_to_half(text):
    return jaconv.z2h(text, digit=True, kana=False)


def alpha_to_full(text):
    return jaconv.alphabet2kana(text)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
from kotonoha import replacer


def test_replace_urls_with_a_text_without_url():
    text = 'This text does not contain any url'
    assert replacer.replace_urls(text) == text


def test_replace_urls_with_a_text_that_contains_an_url():
    text = 'This text contains an url http://abcd.com https://abcd.com'
    assert replacer.replace_urls(text) == 'This text contains an url  '


def test_replace_urls_with_a_different_string():
    text = 'This text contains an url http://abcd.com https://abcd.com'
    assert replacer.replace_urls(text, 'ok') == 'This text contains an url ok ok'


def test_replace_numbers_with_a_text_without_any_number():
    text = 'This text does not contain any number'
    assert replacer.replace_numbers(text) == text


def test_replace_numbers_with_a_text_that_contains_numbers():
    text = 'This text contains numbers 1234 ９２ 0.2 1,000'
    assert replacer.replace_numbers(text) == 'This text contains numbers # # # #,#'


def test_replace_numbers_with_a_different_string():
    text = 'This text contains numbers 1234 ９２ 0.2 1,000'
    assert replacer.replace_numbers(text, 'ok') == 'This text contains numbers ok ok ok ok,ok'


def test_replace_prices_with_a_text_without_any_price():
    text = 'This text does not contain any price 1234'
    assert replacer.replace_prices(text) == text


def test_replace_prices_with_a_text_that_contains_price():
    text = 'This text contains numbers 1234円 ９２円 0.2円 1,000円 ０.２円'
    assert replacer.replace_prices(text) == 'This text contains numbers #円 #円 #円 #円 #円'


def test_replace_prices_with_a_different_string():
    text = 'This text contains numbers 1234円 ９２円 0.2円 1,000円 ０.２円'
    assert replacer.replace_prices(text, 'ok') == 'This text contains numbers ok ok ok ok ok'


def test_replace_with_regex():
    text = 'This number 1234 must be changed'
    my_regular_expression = r'\d+'
    assert replacer.replace_with_regex(text, my_regular_expression) == 'This number  must be changed'


def test_replace_with_regex_and_a_different_string():
    text = 'This number 1234 must be changed'
    my_regular_expression = r'\d+'
    assert replacer.replace_with_regex(text, my_regular_expression, 'ok') == 'This number ok must be changed'


def test_lower():
    text = 'This number 1234 is SO small'
    assert replacer.lower(text) == 'this number 1234 is so small'


def test_replace_hashtags_with_a_text_without_any_hashtag():
    text = 'This text does not contain any hashtag'
    assert replacer.replace_hashtags(text) == text


def test_replace_hashtags_with_a_text_that_contains_hashtags():
    text = 'This text contains hashtags #abc a#aaa #これは'
    assert replacer.replace_hashtags(text) == 'This text contains hashtags a#aaa'


def test_replace_emails_with_a_text_without_any_email():
    text = 'This text does not contain any email'
    assert replacer.replace_emails(text) == text


def test_replace_emails_with_a_text_that_contains_emails():
    text = 'This text contains emails test.test@email.com aa@aa@aa.com aa_as.aa+asd@e.b.c.s'
    assert replacer.replace_emails(text) == 'This text contains emails aa@aa@aa.com'


def test_replace_mentions_with_a_text_without_any_mention():
    text = 'This text does not contain any mention'
    assert replacer.replace_mentions(text) == text


def test_replace_mentions_with_a_text_that_contains_mentions():
    text = 'This text contains mentions @abc @as_aa @as213_asd2 @aa.aa'
    assert replacer.replace_mentions(text) == 'This text contains mentions @aa.aa'
